@@ -1,3 +1,4 @@
+{-# LANGUAGE PackageImports #-}
 {-# LANGUAGE AllowAmbiguousTypes    #-}
 {-# LANGUAGE FlexibleContexts       #-}
 {-# LANGUAGE FlexibleInstances      #-}
@@ -10,7 +11,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Generics.Product.HList
--- Copyright   :  (C) 2019 Csongor Kiss
+-- Copyright   :  (C) 2020 Csongor Kiss
 -- License     :  BSD3
 -- Maintainer  :  Csongor Kiss <kiss.csongor.kiss@gmail.com>
 -- Stability   :  experimental
@@ -24,23 +25,25 @@ module Data.Generics.Product.HList
   ( IsList (..)
   ) where
 
-import Data.Generics.Product.Internal.HList
+import "this" Data.Generics.Internal.VL.Iso (Iso, iso2isovl)
+
+import "generic-lens-core" Data.Generics.Internal.Profunctor.Iso (repIso)
+import qualified "generic-lens-core" Data.Generics.Product.Internal.HList as Core
+
 import Data.Kind
 import GHC.Generics
-import Data.Generics.Internal.VL.Iso (Iso)
-import Data.Generics.Internal.Profunctor.Iso (repIso, iso2isovl)
 
 class IsList
   (f :: Type)
   (g :: Type)
   (as :: [Type])
   (bs :: [Type]) | f -> as, g -> bs where
-  list  :: Iso f g (HList as) (HList bs)
+  list  :: Iso f g (Core.HList as) (Core.HList bs)
 
 instance
   ( Generic f
   , Generic g
-  , GIsList (Rep f) (Rep g) as bs
+  , Core.GIsList (Rep f) (Rep g) as bs
   ) => IsList f g as bs where
-  list = iso2isovl (repIso . glist)
+  list = iso2isovl (repIso . Core.glist)
   {-# INLINE list #-}
